﻿using System.Collections.Generic;

namespace Eingabeverwaltung.Metamodell
{
	public class Sort
	{
		public readonly string name;
		private List<BasicConstructor>    basicConstructors;
		private List<BuildingConstructor> buildingConstructors;

		public Sort(string name)
		{
			this.name = name;
			basicConstructors = new List<BasicConstructor>();
			buildingConstructors = new List<BuildingConstructor>();
		}

		public void SetConstructor(Constructor constructor)
        {
			if (GetConstructors().Contains(constructor)) 
				throw new System.Exception("Tried to add the same constructor "+constructor.name+" twice to the constructors of its sort.");
			if      (constructor is BasicConstructor    basicC)	basicConstructors.Add(basicC);
			else if (constructor is BuildingConstructor buildC)	buildingConstructors.Add(buildC);
			else throw new System.Exception("Found a constructor "+constructor.name+" that is either basic nor building!");
        }

		public List<BasicConstructor> GetBasicConstructors() => new List<BasicConstructor>(basicConstructors);
		public List<BuildingConstructor> GetBuildingConstructors() => new List<BuildingConstructor>(buildingConstructors);
		public List<Constructor> GetConstructors() 
		{
			List<Constructor> result = new List<Constructor>(basicConstructors);
			result.AddRange(buildingConstructors);
			return result; 
		}
	}
}
