﻿using System.Collections.Generic;
using Ueberpruefung;

namespace Eingabeverwaltung.Metamodell
{
	public class TransformationStep : Element
	{
		public readonly Tree startTree;
		public readonly Rule rule;
		public readonly char direction;
		public readonly Tree termPart;
		public readonly Tree resultTree;
		public readonly List<Substitution> substitutions;
		public readonly List<Error>        errors;

		public TransformationStep(Coords coords, Tree startTree, Rule rule, char direction, Tree termPart, Tree resultTree, List<Substitution> substitutions) : base(coords)
		{
			this.startTree = startTree;
			this.rule = rule;
			this.direction = direction;
			this.termPart = termPart;
			this.resultTree = resultTree;
			this.substitutions = substitutions;
			errors = new List<Error>();
		}

		public Tree ruleStartTree()
		{
			switch (direction)
			{
				case '>': return rule.eq.leftTree;
				case '<': return rule.eq.rightTree;
				default: throw new KontextException(coords, "A TransformationStep has an undefined direction: " + direction);
			}
		}
		public Tree ruleEndTree()
		{
			switch (direction)
			{
				case '>': return rule.eq.rightTree;
				case '<': return rule.eq.leftTree;
				default: throw new KontextException(coords, "A TransformationStep has an undefined direction: " + direction);
			}
		}

        public override string ToString()
        {
			string res = "   " + "{ " + rule.name + " , " + direction + " , " + termPart + " , ";
			res += "[ ";
			foreach (Substitution sub in substitutions)	res += sub.ToString();
			res += "]";
			res += " }\n\r";
			res += "= " + resultTree.ToString();
			return res;
		}
    }
}
